import ECharts from 'echarts';
import { addResizeListener } from './dom-resize';

class EChartConfig {
  constructor(id) {
    const dom = document.getElementById(id);
    this.id = id;
    this.dom = dom;
    this.chart = ECharts.init(dom);
    this.option = {
      title: {},
      tooltip: {},
      legend: {},
      xAxis: undefined,
      yAxis: undefined,
      series: undefined,
      geo: undefined,
      grid: {},
    };
  }

  setOption(option) {
    this.option = option;
    return this;
  }

  setSeries(obj) {
    const { option } = this;
    const newSeries = this.option.series.map((item) => ({
      ...item,
      ...obj,
    }));
    this.option = {
      ...option,
      series: newSeries,
    };
    return this;
  }

  setGrid(grid = {}) {
    const { option } = this;
    this.option = {
      ...option,
      grid: {
        ...option.grid,
        ...grid,
      },
    };
    return this;
  }

  setOptDataZoom(dataZoom = {}) {
    const { option } = this;
    this.option = {
      ...option,
      dataZoom: {
        ...option.dataZoom,
        ...dataZoom,
      },
    };
    return this;
  }

  setLegend(legend = {}) {
    const { option } = this;
    this.option = {
      ...option,
      legend: {
        ...option.legend,
        ...legend,
      },
    };
    return this;
  }

  setTitle(title) {
    const { option } = this;
    this.option = {
      ...option,
      title: {
        ...option.title,
        ...title,
      },
    };
    return this;
  }

  draw(callback) {
    const {
      chart,
      option,
      dom,
      title,
    } = this;
    const newOption = {
      ...option,
      title: {
        ...option.title,
        ...title,
      },
    };
    chart.setOption(newOption);
    addResizeListener(dom, () => {
      chart.resize();
      if (typeof callback === 'function') {
        callback();
      }
    });
    return this;
  }

  setClick(func) {
    const { chart } = this;
    chart.off('click');
    chart.on('click', (params) => {
      func(params);
    });
    return this;
  }

  setLineXClick(func) {
    const { chart } = this;
    const zr = chart.getZr();
    if (typeof func === 'function') {
      zr.on('mousemove', (params) => {
        const point = [params.offsetX, params.offsetY];
        if (chart.containPixel({ gridIndex: 0 }, point)) {
          zr.setCursorStyle('pointer');
          return;
        }
        zr.setCursorStyle('default');
      });
      zr.on('mouseout', () => {
        zr.setCursorStyle('default');
      });

      zr.off('click');
      zr.on('click', (params) => {
        const point = [params.offsetX, params.offsetY];
        if (chart.containPixel({ gridIndex: 0 }, point)) {
          const xIndex = chart.convertFromPixel({ seriesIndex: 0 }, point)[0];
          func(Math.abs(xIndex));
        }
      });
    }
    return this;
  }

  setMouseOver(func) {
    const { chart } = this;
    chart.on('mouseover', (params) => {
      func(params);
    });
    return this;
  }

  setMouseOut(func) {
    const { chart } = this;
    chart.on('mouseout', () => {
      func();
    });
    return this;
  }

  setDataZoom(func) {
    const { chart } = this;
    chart.on('georoam', () => {
      func();
    });
    return this;
  }

  clear() {
    const { chart } = this;
    chart.clear();
    return this;
  }

  getPixel(info) {
    const { pos, geo = 'geo' } = info;
    const { lng, lat } = pos;
    const { chart } = this;
    const pixel = chart.convertToPixel(geo, [lng, lat]);
    return pixel;
  }
}
export default EChartConfig;
